@@ -1,4 +1,4 @@
-/* Somaplayer - Copyright (C) 2003-4 bakunin - Andrea Marchesini 
+/* Somaplayer - Copyright (C) 2003-5 bakunin - Andrea Marchesini 
  *                                     <bakunin@autistici.org>
  *
  * This source code is free software; you can redistribute it and/or
@@ -24,11 +24,13 @@
 
 void bad_list_insert (audio *);
 
-#define BAD_LIST_NULL 0
-#define BAD_LIST_WORK 1
-#define BAD_LIST_STOP 2
+#define BAD_LIST_NULL		0
+#define BAD_LIST_WORK		1
+#define BAD_LIST_STOP		2
+#define BAD_LIST_ATTENTION	3
 
-#define BAD_LIST_TIMER 5
+#define BAD_LIST_TIMER		5
+#define BAD_MAX_RETRY		5
 
 #endif
 
