@@ -1,4 +1,4 @@
-/* Somaplayer - Copyright (C) 2003-4 bakunin - Andrea Marchesini 
+/* Somaplayer - Copyright (C) 2003-5 bakunin - Andrea Marchesini 
  *                                     <bakunin@autistici.org>
  *
  * This source code is free software; you can redistribute it and/or
@@ -129,7 +129,7 @@ daemon_option (char *opt)
 	    }
 	  else if (ok)
 	    fatal (_("Error in daemon parameter: %s"), opt);
-	  
+
 	  break;
 
 	case 't':
@@ -178,6 +178,12 @@ daemon_start (void)
     fatal (_("Error: Mutex init."));
 
 #ifdef ENABLE_GTK
+  if (play->graphic && !getenv ("DISPLAY"))
+    {
+      msg_error ("No X server running. Starting in text mode!");
+      play->graphic = 0;
+    }
+
   /* Graphic interface: */
   if (play->graphic)
     if (pthread_create (&th_graphic, NULL, daemon_graphic, NULL))
@@ -410,6 +416,7 @@ daemon_get_client (int fd)
 
   daemon_client *c;
   int port, addr;
+  int id;
 
 #ifdef ENABLE_GTK
   char s[SIZE_BUFFER];
@@ -450,7 +457,7 @@ daemon_get_client (int fd)
 		     (unsigned int) addr % 256, port);
 
 #ifdef ENABLE_GTK
-	      if (!play->graphic)
+	      if (play->graphic)
 		snprintf (s, SIZE_BUFFER, "%d.%d.%d.%d",
 			  (unsigned int) addr >> 24,
 			  (unsigned int) (addr >> 16) % 256,
@@ -479,7 +486,7 @@ daemon_get_client (int fd)
 		     (unsigned int) addr % 256, port);
 
 #ifdef ENABLE_GTK
-	      if (!play->graphic)
+	      if (play->graphic)
 		snprintf (s, SIZE_BUFFER, "%d.%d.%d.%d",
 			  (unsigned int) addr >> 24,
 			  (unsigned int) (addr >> 16) % 256,
@@ -506,10 +513,9 @@ daemon_get_client (int fd)
 	    msg (_("New client request"));
 
 #ifdef ENABLE_GTK
-	  if (!play->graphic)
+	  if (play->graphic)
 	    snprintf (s, SIZE_BUFFER, "Unix Socket");
 #endif
-	  strcpy (s, "socket");
 	}
 
       /* In the first line there is the type of clients. 
@@ -544,7 +550,7 @@ daemon_get_client (int fd)
 	}
 
       /* For test check */
-      if (!strncmp (buffer, "TEST", 4))
+      if (!strncmp (buffer, SDS_TEST_CONNECT, strlen(SDS_TEST_CONNECT)))
 	{
 	  close (c->fd);
 
@@ -554,13 +560,49 @@ daemon_get_client (int fd)
 	  continue;
 	}
 
+      else if (!strncmp (buffer, SDS_DATA_LIST_CONNECT, strlen(SDS_DATA_LIST_CONNECT)))
+	{
+	  int i, len;
+	  char buf[SIZE_BUFFER];
+
+	  if (!play->noverbose)
+	    msg (_("Connection data list request"));
+
+	  pthread_mutex_lock (&play->daemon->mutex);
+
+          for(i=0;i<DAEMON_MAX_CLIENTS;i++) {
+            if(play->daemon->client[i]!=NULL) {
+	    len=snprintf(buf, sizeof(buf), "%s %d %d %d %d %d %d %d %d %d %s\n", 
+			    play->daemon->client[i]->addr, 
+			    play->daemon->client[i]->mute, 
+			    play->daemon->client[i]->solo, 
+			    play->daemon->client[i]->v_up, 
+			    play->daemon->client[i]->v_down, 
+			    play->daemon->client[i]->b_up, 
+			    play->daemon->client[i]->b_down, 
+			    play->daemon->client[i]->volume, 
+			    play->daemon->client[i]->balance, 
+			    play->daemon->client[i]->autofader,
+			    play->daemon->client[i]->socket
+			    );
+	    write(c->fd, buf, len);
+	    }
+	  }
+
+	  pthread_mutex_unlock (&play->daemon->mutex);
+
+	  close (c->fd);
+
+	  continue;
+	}
+
       /* For me a monitor is ONLY another output. So I add in
        * the list of outputs this new element. */
-      else if (!strncmp (buffer, "MONITOR ", 8))
+      else if (!strncmp (buffer, SDS_MONITOR_CONNECT, strlen(SDS_MONITOR_CONNECT)))
 	{
 	  audio *tmp;
 
-	  sscanf (buffer + 8, "%d", &c->bigendian);
+	  sscanf (buffer + strlen(SDS_MONITOR_CONNECT), "%d", &c->bigendian);
 
 	  if (!(tmp = (audio *) malloc (sizeof (audio))))
 	    fatal (_("Error: memory."));
@@ -574,14 +616,15 @@ daemon_get_client (int fd)
 	  tmp->close = monitor_close;
 	  tmp->check = NULL;
 	  tmp->info = monitor_show;
+	  tmp->update = NULL;
 	  tmp->data = (void *) c;
 	  tmp->next = NULL;
 	  tmp->status = OUTPUT_DONE;
 
+#ifdef ENABLE_GTK
 	  if (!(tmp->audio_dev = strdup (s)))
 	    fatal ("Error: memory.");
 
-#ifdef ENABLE_GTK
 	  tmp->stop = 0;
 #endif
 
@@ -614,7 +657,7 @@ daemon_get_client (int fd)
 
 #ifdef ENABLE_GTK
 		  if (play->graphic)
-		    output_window_add (tmp, PREPEND);
+		    output_list_add (tmp, PREPEND);
 #endif
 		}
 	    }
@@ -629,7 +672,7 @@ daemon_get_client (int fd)
 
 #ifdef ENABLE_GTK
 	      if (play->graphic)
-		output_window_add (tmp, PREPEND);
+		output_list_add (tmp, PREPEND);
 #endif
 	    }
 
@@ -644,6 +687,21 @@ daemon_get_client (int fd)
 	sscanf (buffer, "%d %d %d %d %d %d %d %d", &c->rate, &c->channels,
 		&c->bitrate, &c->volume, &c->balance, &c->solo, &c->mute,
 		&c->bigendian);
+
+      c->socket=NULL;
+
+      for(id=strlen(buffer)-1;id;id--) {
+        if(buffer[id]==' ') {
+		id++;
+		if(!strncmp(buffer+id, "unix://",7) && strlen(buffer+id)>7) 
+			c->socket=strdup(buffer+id);
+		else if(!strncmp(buffer+id, "tcp://",6) && strlen(buffer+id)>6)
+			c->socket=strdup(buffer+id);
+
+		break;
+	}
+      }
+
 #else
       ret = sscanf (buffer, "%d %d %d", &c->rate, &c->channels, &c->bitrate);
 #endif
@@ -685,6 +743,7 @@ daemon_get_client (int fd)
 	case 6:
 	  c->mute = 0;
 	  break;
+
 	}
 
 #endif
@@ -747,6 +806,8 @@ daemon_get_client (int fd)
 #ifdef ENABLE_GTK
       if (play->graphic)
 	{
+	  if(!(c->addr = strdup(s)))
+		  fatal(_("Error: memory."));
 	  c->widget = gdaemon_add_mix (s, c);
 	  msg_gstatusbar (_("Connection from %s"), s);
 	}
@@ -796,6 +857,7 @@ daemon_play (void *a)
 	   */
 	  if (play->daemon->client[i] != NULL)
 	    {
+	      n_inputs = 1;
 
 	      /* If the client exist and the buffer is empty, it is exist? */
 	      if (play->daemon->client[i]->done == 1)
@@ -814,6 +876,7 @@ daemon_play (void *a)
 		    }
 #endif
 
+		  close(play->daemon->client[i]->fd);
 		  free (play->daemon->client[i]);
 		  play->daemon->client[i] = NULL;
 
@@ -834,8 +897,6 @@ daemon_play (void *a)
 		      (&play->daemon->client[i]->mutex) != EBUSY)
 		    {
 
-		      n_inputs++;
-
 #ifdef ENABLE_GTK
 		      /* In graphic i can change the single volumes, so i change
 		       * it here: */
@@ -860,11 +921,6 @@ daemon_play (void *a)
 			    buffer[k] += p[k];
 #ifdef ENABLE_GTK
 			}
-		      else if (n_inputs == 1)
-			{
-			  /* if all is mute ... */
-			  memset (buffer, 0, SIZE_BUFFER);
-			}
 #endif
 		      /* Ok, now i must erase one element of this client: */
 		      if (play->daemon->client[i]->id_r ==
@@ -1132,17 +1188,15 @@ void
 daemon_check_bigendian (daemon_client * c, char *bug, int len)
 {
   int i;
-  char t;
+  int16_t *k = (int16_t *) bug;
 
   if (c->bigendian == big_endian ())
     return;
 
-  for (i = 0; i < len / 2; i += 2)
-    {
-      t = bug[i];
-      bug[i] = bug[i + 1];
-      bug[i + 1] = t;
-    }
+  for (i = 0; i < len / 2; i++)
+    k[i] =
+      (((u_int16_t) k[i] & 0x00ffU) << 8 |
+       (((u_int16_t) k[i] & 0xff00U) >> 8));
 }
 
 #ifdef ENABLE_GTK
