@@ -1,4 +1,4 @@
-/* Somaplayer - Copyright (C) 2003-4 bakunin - Andrea Marchesini 
+/* Somaplayer - Copyright (C) 2003-5 bakunin - Andrea Marchesini 
  *                                     <bakunin@autistici.org>
  *
  * This source code is free software; you can redistribute it and/or
@@ -32,8 +32,20 @@
 #include "gtkled.h"
 #include "gtkledbar.h"
 
+#ifdef ENABLE_SOMALIST
+#include <somalist.h>
+#endif
+
 gtk_element_input *element_input;
 
+GtkTargetEntry targetentries[] = {
+  {"STRING", 0, 0}
+  ,
+  {"text/plain", 0, 0}
+  ,
+  {"application/x-rootwindow-drop", 0, 1}
+};
+
 GtkWidget *
 create_winmain (void)
 {
@@ -64,12 +76,19 @@ create_winmain (void)
 
   GtkWidget *output;
   GtkWidget *output_menu;
+  GtkWidget *output_menu_s;
   GtkWidget *output_item;
 
   GtkWidget *m_help;
   GtkWidget *m_help_menu;
   GtkWidget *about;
 
+#ifdef ENABLE_SOMALIST
+  GtkWidget *playmenu;
+  GtkWidget *playmenu_menu;
+  GtkWidget *playmenu_item;
+#endif
+
   GtkWidget *tools;
   GtkWidget *tools_menu;
   GtkWidget *tools_item;
@@ -104,15 +123,25 @@ create_winmain (void)
   GtkWidget *add_m;
   GtkWidget *playlist_m;
 
+  GtkWidget *output_list_sw;
+  GtkWidget *output_list_m;
+  GtkTreeSelection *selection;
+  GtkTreeSelection *p_selection;
+
   char s[SIZE_BUFFER];
+  struct stat st;
 
   accel_group = gtk_accel_group_new ();
 
+  output_list_sw = create_output_list ();
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (output_list));
+
   winmain = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   sprintf (s, "%s - %s", NAME, VERSION);
   gtk_window_set_title (GTK_WINDOW (winmain), s);
   gtk_window_set_default_size (GTK_WINDOW (winmain), play->size_x,
 			       play->size_y);
+  gtk_window_set_default_icon_from_file(SOMAPLAYER_ICON, NULL);
 
   vbox = gtk_vbox_new (FALSE, 0);
   gtk_widget_show (vbox);
@@ -309,24 +338,17 @@ create_winmain (void)
   output_menu = gtk_menu_new ();
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (output), output_menu);
 
-  output_item = gtk_menu_item_new_with_mnemonic (_("_Show Outputs"));
+  output_item = gtk_menu_item_new_with_mnemonic (_("_Add"));
   gtk_widget_show (output_item);
   gtk_container_add (GTK_CONTAINER (output_menu), output_item);
 
-  g_signal_connect ((gpointer) output_item, "activate",
-		    G_CALLBACK (output_window_show), NULL);
-
-  output_item = gtk_menu_item_new_with_mnemonic (_("_Add Output"));
-  gtk_widget_show (output_item);
-  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
-
-  output_menu = gtk_menu_new ();
-  gtk_menu_item_set_submenu (GTK_MENU_ITEM (output_item), output_menu);
+  output_menu_s = gtk_menu_new ();
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (output_item), output_menu_s);
 
 #ifdef ENABLE_AO
   output_item = gtk_menu_item_new_with_mnemonic (_("Audio"));
   gtk_widget_show (output_item);
-  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu_s), output_item);
 
   g_signal_connect ((gpointer) output_item, "activate",
 		    G_CALLBACK (on_ao_new_activate), NULL);
@@ -336,7 +358,7 @@ create_winmain (void)
   output_item =
     gtk_menu_item_new_with_mnemonic (_("Soma Distribuited Sound"));
   gtk_widget_show (output_item);
-  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu_s), output_item);
 
   g_signal_connect ((gpointer) output_item, "activate",
 		    G_CALLBACK (on_sds_new_activate), NULL);
@@ -345,7 +367,7 @@ create_winmain (void)
 #ifdef ENABLE_LAME
   output_item = gtk_menu_item_new_with_mnemonic (_("Encode Mp3"));
   gtk_widget_show (output_item);
-  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu_s), output_item);
 
   g_signal_connect ((gpointer) output_item, "activate",
 		    G_CALLBACK (on_lame_new_activate), NULL);
@@ -354,7 +376,7 @@ create_winmain (void)
 #ifdef ENABLE_OGG
   output_item = gtk_menu_item_new_with_mnemonic (_("Encode Ogg"));
   gtk_widget_show (output_item);
-  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu_s), output_item);
 
   g_signal_connect ((gpointer) output_item, "activate",
 		    G_CALLBACK (on_ogg_new_activate), NULL);
@@ -363,7 +385,7 @@ create_winmain (void)
 #ifdef ENABLE_SNDFILE
   output_item = gtk_menu_item_new_with_mnemonic (_("Write a file"));
   gtk_widget_show (output_item);
-  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu_s), output_item);
 
   g_signal_connect ((gpointer) output_item, "activate",
 		    G_CALLBACK (on_write_a_file_new_activate), NULL);
@@ -374,7 +396,7 @@ create_winmain (void)
 #ifdef ENABLE_ICECAST
   output_item = gtk_menu_item_new_with_mnemonic (_("Icecast"));
   gtk_widget_show (output_item);
-  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu_s), output_item);
 
   g_signal_connect ((gpointer) output_item, "activate",
 		    G_CALLBACK (on_icecast_new_activate), NULL);
@@ -383,7 +405,7 @@ create_winmain (void)
 #ifdef ENABLE_ICECAST2
   output_item = gtk_menu_item_new_with_mnemonic (_("Icecast2"));
   gtk_widget_show (output_item);
-  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu_s), output_item);
 
   g_signal_connect ((gpointer) output_item, "activate",
 		    G_CALLBACK (on_icecast2_new_activate), NULL);
@@ -392,12 +414,49 @@ create_winmain (void)
 #ifdef ENABLE_SHOUTCAST
   output_item = gtk_menu_item_new_with_mnemonic (_("Shoutcast"));
   gtk_widget_show (output_item);
-  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu_s), output_item);
 
   g_signal_connect ((gpointer) output_item, "activate",
 		    G_CALLBACK (on_shoutcast_new_activate), NULL);
 #endif
 
+#endif
+
+  output_item = gtk_menu_item_new_with_mnemonic (_("_Remove"));
+  gtk_widget_show (output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+
+  g_signal_connect ((gpointer) output_item, "activate",
+		    G_CALLBACK (output_list_select_remove), selection);
+
+  output_item = gtk_menu_item_new_with_mnemonic (_("_Show"));
+  gtk_widget_show (output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+
+  g_signal_connect ((gpointer) output_item, "activate",
+		    G_CALLBACK (output_list_select_show), selection);
+
+  output_item = gtk_menu_item_new_with_mnemonic (_("S_top/Start"));
+  gtk_widget_show (output_item);
+  gtk_container_add (GTK_CONTAINER (output_menu), output_item);
+
+  g_signal_connect ((gpointer) output_item, "activate",
+		    G_CALLBACK (output_list_select_stop), selection);
+
+#ifdef ENABLE_SOMALIST
+  playmenu = gtk_menu_item_new_with_mnemonic (_("_Playlist"));
+  gtk_widget_show (playmenu);
+  gtk_container_add (GTK_CONTAINER (menubar), playmenu);
+
+  playmenu_menu = gtk_menu_new ();
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (playmenu), playmenu_menu);
+
+  playmenu_item = gtk_menu_item_new_with_mnemonic (_("_Show Playlist Admin"));
+  gtk_widget_show (playmenu_item);
+  gtk_container_add (GTK_CONTAINER (playmenu_menu), playmenu_item);
+
+  g_signal_connect ((gpointer) playmenu_item, "activate",
+		    G_CALLBACK (spls_window_show), NULL);
 #endif
 
   tools = gtk_menu_item_new_with_mnemonic (_("_Tools"));
@@ -656,7 +715,7 @@ create_winmain (void)
 
   scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_show (scrolledwindow);
-  gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow, FALSE, FALSE, 0);
   gtk_widget_set_size_request (scrolledwindow, 190, 80);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
 				  GTK_POLICY_NEVER, GTK_POLICY_NEVER);
@@ -668,6 +727,8 @@ create_winmain (void)
   gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textarea), FALSE);
   text = (GtkWidget *) gtk_text_view_get_buffer (GTK_TEXT_VIEW (textarea));
 
+  gtk_box_pack_start (GTK_BOX (vbox2), output_list_sw, TRUE, TRUE, 0);
+
   hbox_bottom_dx = gtk_hbox_new (FALSE, 0);
   gtk_widget_show (hbox_bottom_dx);
   gtk_table_attach (GTK_TABLE (table), hbox_bottom_dx, 4, 5, 1, 2,
@@ -728,13 +789,20 @@ create_winmain (void)
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
-  model = gtk_list_store_new (1, G_TYPE_STRING);
+  model =
+    gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING);
   playlist =
     (GtkWidget *) gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
+  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (playlist), FALSE);
   p_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (playlist));
   gtk_tree_selection_set_mode (p_selection, GTK_SELECTION_MULTIPLE);
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (playlist), TRUE);
 
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (playlist), -1,
+					       _("Q"), renderer,
+					       "text", 2, NULL);
+
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (playlist), -1,
 					       _("Playlist"), renderer,
@@ -753,6 +821,7 @@ create_winmain (void)
   gtk_widget_show (statusbar);
   gtk_box_pack_start (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);
 
+  output_list_m = create_output_list_menu (selection);
   add_m = add_menu ();
   playlist_m = playlist_menu ();
 
@@ -796,9 +865,12 @@ create_winmain (void)
 		    (gtk_element_input *) change);
 #endif
 
+  gtk_signal_connect_object (GTK_OBJECT (output_list), "button_press_event",
+			     GTK_SIGNAL_FUNC (output_list_m_show),
+			     output_list_m);
+
   g_signal_connect ((gpointer) about, "activate",
 		    G_CALLBACK (on_about_activate), NULL);
-
   g_signal_connect ((gpointer) b_prev, "clicked",
 		    G_CALLBACK (on_b_prev_clicked), NULL);
   g_signal_connect ((gpointer) b_play, "clicked",
@@ -815,13 +887,38 @@ create_winmain (void)
 		    G_CALLBACK (on_b_random_toggled), NULL);
   g_signal_connect ((gpointer) b_repeat, "toggled",
 		    G_CALLBACK (on_b_repeat_toggled), NULL);
-  gtk_signal_connect_object (GTK_OBJECT (b_add), "button_press_event",
-			     GTK_SIGNAL_FUNC (add_menu_popup), add_m);
-  gtk_signal_connect_object (GTK_OBJECT (playlist), "button_press_event",
-			     GTK_SIGNAL_FUNC (playlist_menu_popup),
-			     playlist_m);
   g_signal_connect ((gpointer) b_remove, "clicked",
 		    G_CALLBACK (on_b_remove_clicked), p_selection);
+  g_signal_connect (GTK_OBJECT (b_add), "button_press_event",
+			     GTK_SIGNAL_FUNC (add_menu_popup), add_m);
+  g_signal_connect (GTK_OBJECT (playlist), "button-press-event",
+		    GTK_SIGNAL_FUNC (playlist_menu_popup), playlist_m);
+
+  g_signal_connect ((gpointer) playlist, "drag_data_get",
+		    G_CALLBACK (playlist_drag_get), NULL);
+  g_signal_connect ((gpointer) playlist, "drag_data_received",
+		    G_CALLBACK (playlist_drop), NULL);
+  g_signal_connect ((gpointer) playlist, "drag_begin",
+		    G_CALLBACK (playlist_drag_begin), NULL);
+  g_signal_connect ((gpointer) playlist, "drag_end",
+		    G_CALLBACK (playlist_drag_end), NULL);
+
+  gtk_drag_source_set (playlist, GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
+		       targetentries, 3, GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
+  gtk_drag_dest_set (playlist, GTK_DEST_DEFAULT_ALL, targetentries, 3,
+		     GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
+  g_signal_connect ((gpointer) winmain, "key-press-event",
+		    GTK_SIGNAL_FUNC (playlist_key_event), p_selection);
+
+  if (!lstat (SOMAPLAYER_ICON, &st))
+    {
+      GdkPixbuf *pb;
+
+      pb = gdk_pixbuf_new_from_file (SOMAPLAYER_ICON, NULL);
+      gtk_drag_source_set_icon_pixbuf (playlist, pb);
+    }
 
   gtk_window_add_accel_group (GTK_WINDOW (winmain), accel_group);
 
