@@ -1,4 +1,4 @@
-/* Somaplayer - Copyright (C) 2003-4 bakunin - Andrea Marchesini 
+/* Somaplayer - Copyright (C) 2003-5 bakunin - Andrea Marchesini 
  *                                     <bakunin@autistici.org>
  *
  * This source code is free software; you can redistribute it and/or
@@ -32,10 +32,10 @@
 
 static int output_win = 1;
 
-gint output_window_popup (GtkWidget *, GdkEvent *);
+gint output_list_popup (GtkWidget *, GdkEvent *, gpointer);
 
 void
-output_window_select_stop (GtkWidget * w, GtkTreeSelection * selection)
+output_list_select_stop (GtkWidget * w, GtkTreeSelection * selection)
 {
 
   GtkTreeModel *model;
@@ -58,7 +58,7 @@ output_window_select_stop (GtkWidget * w, GtkTreeSelection * selection)
       else
 	ao->stop = 1;
 
-      output_window_refresh ();
+      output_list_refresh ();
 
       gtk_tree_selection_unselect_all (selection);
       gtk_tree_selection_select_iter (selection, &iter);
@@ -66,7 +66,7 @@ output_window_select_stop (GtkWidget * w, GtkTreeSelection * selection)
 }
 
 void
-output_window_select_show (GtkWidget * w, GtkTreeSelection * selection)
+output_list_select_show (GtkWidget * w, GtkTreeSelection * selection)
 {
 
   GtkTreeModel *model;
@@ -174,7 +174,7 @@ output_window_select_show (GtkWidget * w, GtkTreeSelection * selection)
 }
 
 void
-output_window_select_remove (GtkWidget * w, GtkTreeSelection * selection)
+output_list_select_remove (GtkWidget * w, GtkTreeSelection * selection)
 {
 
   GtkTreeModel *model;
@@ -186,8 +186,7 @@ output_window_select_remove (GtkWidget * w, GtkTreeSelection * selection)
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
 
-      if (gtk_dialog_ask (_("Sure to delete this output?")) ==
-	  GTK_RESPONSE_OK)
+      if (dialog_ask (_("Sure to delete this output?")) == GTK_RESPONSE_OK)
 	{
 	  audio *ao;
 
@@ -228,7 +227,7 @@ output_window_select_remove (GtkWidget * w, GtkTreeSelection * selection)
 }
 
 void
-output_window_add (audio * ao, int where)
+output_list_add (audio * ao, int where)
 {
   GtkTreeView *view = GTK_TREE_VIEW (output_list);
   GtkTreeModel *model;
@@ -545,50 +544,85 @@ output_window_add (audio * ao, int where)
 }
 
 void
-output_window_refresh (void)
+output_list_refresh (void)
 {
   GtkTreeModel *model;
   GtkTreeIter iter;
   audio *ao;
 
+  output_lock ();
+
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (output_list));
   while (gtk_tree_model_iter_nth_child
 	 (GTK_TREE_MODEL (model), &iter, NULL, 0))
     gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
 
-  output_lock ();
-
   ao = play->output;
   while (ao)
     {
-      output_window_add (ao, APPEND);
+      output_list_add (ao, APPEND);
       ao = ao->next;
     }
 
   output_unlock ();
 }
 
-int
-output_window_show (GtkWidget * w, gpointer data)
+gboolean
+output_list_m_show (GtkWidget * w, GdkEventButton * event, gpointer menu)
 {
-
-  if (data || !output_win)
+  if (event->button == 3)
     {
-      output_win = 1;
-      gtk_widget_hide (winoutput);
-    }
-  else if (output_win)
-    {
-      output_window_refresh ();
-      gtk_widget_show (winoutput);
-      output_win = 0;
+      gtk_menu_popup (GTK_MENU (w), NULL, NULL, NULL, NULL, event->button,
+		      event->time);
+      return TRUE;
     }
 
-  return TRUE;
+  return FALSE;
 }
 
 GtkWidget *
-output_window_menu (void)
+create_output_list_menu (GtkTreeSelection * selection)
+{
+  GtkWidget *menu;
+  GtkWidget *menu_s;
+  GtkWidget *item;
+
+  menu = gtk_menu_new ();
+
+  item = gtk_menu_item_new_with_mnemonic (_("Add"));
+  gtk_widget_show (item);
+  gtk_container_add (GTK_CONTAINER (menu), item);
+
+  menu_s = output_list_menu ();
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu_s);
+
+  item = gtk_menu_item_new_with_mnemonic (_("Remove"));
+  gtk_widget_show (item);
+  gtk_container_add (GTK_CONTAINER (menu), item);
+
+  g_signal_connect ((gpointer) item, "activate",
+		    G_CALLBACK (output_list_select_remove), selection);
+
+  item = gtk_menu_item_new_with_mnemonic (_("Show"));
+  gtk_widget_show (item);
+  gtk_container_add (GTK_CONTAINER (menu), item);
+
+  g_signal_connect ((gpointer) item, "activate",
+		    G_CALLBACK (output_list_select_show), selection);
+
+  item = gtk_menu_item_new_with_mnemonic (_("Stop/Start"));
+  gtk_widget_show (item);
+  gtk_container_add (GTK_CONTAINER (menu), item);
+
+  g_signal_connect ((gpointer) item, "activate",
+		    G_CALLBACK (output_list_select_stop), selection);
+
+
+  return menu;
+}
+
+GtkWidget *
+output_list_menu (void)
 {
   GtkWidget *menu;
   GtkWidget *audio;
@@ -704,15 +738,14 @@ output_window_menu (void)
 }
 
 gint
-output_window_popup (GtkWidget * widget, GdkEvent * event)
+output_list_popup (GtkWidget * widget, GdkEvent * event, gpointer menu)
 {
 
   if (event->type == GDK_BUTTON_PRESS)
     {
       GdkEventButton *bevent = (GdkEventButton *) event;
 
-      gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL,
-		      bevent->button, bevent->time);
+      gtk_menu_popup (GTK_MENU (menu), NULL, NULL, popup_position, widget, bevent->button, bevent->time);
       return TRUE;
     }
 
@@ -720,46 +753,22 @@ output_window_popup (GtkWidget * widget, GdkEvent * event)
 }
 
 GtkWidget *
-create_winoutput (void)
+create_output_list (void)
 {
-  GtkWidget *window;
-  GtkWidget *hbox;
   GtkWidget *scrolledwindow;
-
-  GtkTreeSelection *selection;
   GtkListStore *model;
   GtkCellRenderer *renderer;
-
-  GtkWidget *vbox;
-  GtkWidget *add;
-  GtkWidget *alignment;
-  GtkWidget *image;
-  GtkWidget *label;
-  GtkWidget *remove;
-  GtkWidget *show;
-  GtkWidget *stop;
-
-  GtkWidget *menu = output_window_menu ();
-
-  char s[SIZE_BUFFER];
-
-  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  sprintf (s, _("%s %s - Output Window"), NAME, VERSION);
-  gtk_window_set_title (GTK_WINDOW (window), s);
-
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_widget_show (hbox);
-  gtk_container_add (GTK_CONTAINER (window), hbox);
+  GtkTreeSelection *selection;
 
   scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_show (scrolledwindow);
-  gtk_box_pack_start (GTK_BOX (hbox), scrolledwindow, TRUE, TRUE, 0);
   gtk_widget_set_size_request (scrolledwindow, 250, -1);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
   model = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);
   output_list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
+  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (output_list), FALSE);
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (output_list));
   gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (output_list), TRUE);
@@ -775,16 +784,84 @@ create_winoutput (void)
 
   gtk_container_add (GTK_CONTAINER (scrolledwindow), output_list);
 
+  return scrolledwindow;
+}
+
+int
+output_window_show (GtkWidget * w, gpointer data)
+{
+
+  if (data || !output_win)
+    {
+      output_win = 1;
+      gtk_widget_hide (winoutput);
+    }
+  else if (output_win)
+    {
+      output_list_refresh ();
+      gtk_widget_show (winoutput);
+      output_win = 0;
+    }
+
+  return TRUE;
+}
+
+gboolean
+output_key_event (GtkWidget * w, GdkEventKey * event, gpointer dummy)
+{
+  if (event->keyval == GDK_Escape)
+    {
+      output_window_show (NULL, NULL);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+GtkWidget *
+create_winoutput (void)
+{
+  GtkWidget *window;
+  GtkWidget *box;
+  GtkWidget *hbox;
+
+  GtkWidget *vbox;
+  GtkWidget *add;
+  GtkWidget *alignment;
+  GtkWidget *image;
+  GtkWidget *label;
+  GtkWidget *remove;
+  GtkWidget *show;
+  GtkWidget *stop;
+
+  GtkWidget *scrolledwindow;
+  GtkTreeSelection *selection;
+
+  char s[SIZE_BUFFER];
+
+  GtkWidget *menu = output_list_menu ();
+
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  sprintf (s, _("%s %s - Output Window"), NAME, VERSION);
+  gtk_window_set_title (GTK_WINDOW (window), s);
+
+  box = gtk_hbox_new (FALSE, 0);
+  gtk_widget_show (box);
+  gtk_container_add (GTK_CONTAINER (window), box);
+
+  scrolledwindow = create_output_list ();
+  gtk_box_pack_start (GTK_BOX (box), scrolledwindow, TRUE, TRUE, 0);
+
   vbox = gtk_vbox_new (FALSE, 0);
   gtk_widget_show (vbox);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (box), vbox, TRUE, TRUE, 0);
 
   add = gtk_button_new ();
   gtk_widget_show (add);
   gtk_box_pack_start (GTK_BOX (vbox), add, FALSE, FALSE, 0);
 
-  gtk_signal_connect_object (GTK_OBJECT (add), "event",
-			     GTK_SIGNAL_FUNC (output_window_popup), menu);
+  g_signal_connect (GTK_OBJECT (add), "event",
+			     GTK_SIGNAL_FUNC (output_list_popup), menu);
 
   alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
   gtk_widget_show (alignment);
@@ -822,8 +899,10 @@ create_winoutput (void)
   gtk_widget_show (label);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (output_list));
+
   g_signal_connect ((gpointer) remove, "clicked",
-		    G_CALLBACK (output_window_select_remove), selection);
+		    G_CALLBACK (output_list_select_remove), selection);
 
   show = gtk_button_new ();
   gtk_widget_show (show);
@@ -846,7 +925,7 @@ create_winoutput (void)
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
   g_signal_connect ((gpointer) show, "clicked",
-		    G_CALLBACK (output_window_select_show), selection);
+		    G_CALLBACK (output_list_select_show), selection);
 
   stop = gtk_button_new ();
   gtk_widget_show (stop);
@@ -869,11 +948,14 @@ create_winoutput (void)
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
   g_signal_connect ((gpointer) stop, "clicked",
-		    G_CALLBACK (output_window_select_stop), selection);
+		    G_CALLBACK (output_list_select_stop), selection);
 
   g_signal_connect ((gpointer) window, "delete_event",
 		    G_CALLBACK (output_window_show), window);
 
+  g_signal_connect ((gpointer) window, "key-press-event",
+		    GTK_SIGNAL_FUNC (output_key_event), NULL);
+
   return window;
 }
 
