@@ -1,4 +1,4 @@
-/* Somaplayer - Copyright (C) 2003-4 bakunin - Andrea Marchesini 
+/* Somaplayer - Copyright (C) 2003-5 bakunin - Andrea Marchesini 
  *                                     <bakunin@autistici.org>
  *
  * This source code is free software; you can redistribute it and/or
@@ -224,19 +224,19 @@ on_shoutcast_new_activate (GtkMenuItem * menuitem, gpointer user_data)
 	      (gtk_entry_get_text (GTK_ENTRY (password)),
 	       gtk_entry_get_text (GTK_ENTRY (password2))))
 	    {
-	      gtk_dialog_msg (_("Passwords are different!"));
+	      dialog_msg (_("Passwords are different!"));
 	      continue;
 	    }
 
 	  if (!strlen (gtk_entry_get_text (GTK_ENTRY (server))))
 	    {
-	      gtk_dialog_msg (_("No server?"));
+	      dialog_msg (_("No server?"));
 	      continue;
 	    }
 
 	  if (!strlen (gtk_entry_get_text (GTK_ENTRY (mount))))
 	    {
-	      gtk_dialog_msg (_("No mount?"));
+	      dialog_msg (_("No mount?"));
 	      continue;
 	    }
 
@@ -315,6 +315,7 @@ on_shoutcast_new_activate (GtkMenuItem * menuitem, gpointer user_data)
 	  tmp->open = shoutcast_start;
 	  tmp->write = streaming_write;
 	  tmp->close = streaming_quit;
+	  tmp->update = NULL;
 	  tmp->check = NULL;
 	  tmp->info = streaming_show;
 	  tmp->data = (void *) l;
@@ -351,7 +352,7 @@ on_shoutcast_new_activate (GtkMenuItem * menuitem, gpointer user_data)
 
 		  output_unlock ();
 
-		  output_window_add (tmp, PREPEND);
+		  output_list_add (tmp, PREPEND);
 		}
 	    }
 	  else
@@ -363,40 +364,25 @@ on_shoutcast_new_activate (GtkMenuItem * menuitem, gpointer user_data)
 
 	      output_unlock ();
 
-	      output_window_add (tmp, PREPEND);
+	      output_list_add (tmp, PREPEND);
 	    }
 	  break;
 	}
 
-      /* Cancel button */
-      else if (ret == GTK_RESPONSE_CANCEL)
-	{
-	  break;
-	}
-
       /* Open button */
       else if (ret == 100)
 	{
 	  audio_data_streaming *data;
-
-	  GtkWidget *fs;
+	  char *file;
 	  char s[1024];
 
 	  snprintf (s, 1024, "%s %s - Open shoutcast config file...", NAME,
 		    VERSION);
-	  fs =
-	    gtk_file_chooser_dialog_new (s, GTK_WINDOW (winmain),
-					 GTK_FILE_CHOOSER_ACTION_OPEN,
-					 GTK_STOCK_CANCEL,
-					 GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN,
-					 GTK_RESPONSE_OK, NULL);
 
-	  if (gtk_dialog_run (GTK_DIALOG (fs)) == GTK_RESPONSE_OK)
+	  if ((file =
+	       file_chooser (s, GTK_SELECTION_SINGLE,
+			     GTK_FILE_CHOOSER_ACTION_OPEN)))
 	    {
-	      char *file =
-		(char *)
-		gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fs));
-
 	      if (!
 		  (data =
 		   (audio_data_streaming *)
@@ -433,33 +419,44 @@ on_shoutcast_new_activate (GtkMenuItem * menuitem, gpointer user_data)
 		      gtk_combo_box_set_active (GTK_COMBO_BOX (channels), 0);
 		    }
 
-		  gtk_entry_set_text (GTK_ENTRY (password), data->password);
-		  gtk_entry_set_text (GTK_ENTRY (password2), data->password);
+		  gtk_entry_set_text (GTK_ENTRY (password),
+				      data->password ? data->password : "");
+		  gtk_entry_set_text (GTK_ENTRY (password2),
+				      data->password ? data->password : "");
 
-		  gtk_entry_set_text (GTK_ENTRY (server), data->server);
+		  gtk_entry_set_text (GTK_ENTRY (server),
+				      data->server ? data->server : "");
 
 		  gtk_spin_button_set_value (GTK_SPIN_BUTTON (port),
 					     (gdouble) data->port);
 
-		  gtk_entry_set_text (GTK_ENTRY (mount), data->mount);
+		  gtk_entry_set_text (GTK_ENTRY (mount),
+				      data->mount ? data->mount : "");
 
-		  gtk_entry_set_text (GTK_ENTRY (genre), data->genre);
+		  gtk_entry_set_text (GTK_ENTRY (genre),
+				      data->genre ? data->genre : "");
 
-		  gtk_entry_set_text (GTK_ENTRY (name), data->name);
+		  gtk_entry_set_text (GTK_ENTRY (name),
+				      data->name ? data->name : "");
 
-		  gtk_entry_set_text (GTK_ENTRY (aim), data->aim);
+		  gtk_entry_set_text (GTK_ENTRY (aim),
+				      data->aim ? data->aim : "");
 
-		  gtk_entry_set_text (GTK_ENTRY (irc), data->irc);
+		  gtk_entry_set_text (GTK_ENTRY (irc),
+				      data->irc ? data->irc : "");
 
-		  gtk_entry_set_text (GTK_ENTRY (icq), data->icq);
+		  gtk_entry_set_text (GTK_ENTRY (icq),
+				      data->icq ? data->icq : "");
 
-		  gtk_entry_set_text (GTK_ENTRY (url), data->url);
+		  gtk_entry_set_text (GTK_ENTRY (url),
+				      data->url ? data->url : "");
 
 		  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (public),
 						data->public);
 
 		  gtk_entry_set_text (GTK_ENTRY (dumpfilelocal),
-				      data->dumpfilelocal);
+				      data->dumpfilelocal ? data->
+				      dumpfilelocal : "");
 
 		  if (data->dumpfileremote)
 		    free (data->dumpfileremote);
@@ -502,41 +499,30 @@ on_shoutcast_new_activate (GtkMenuItem * menuitem, gpointer user_data)
 	      free (file);
 	      free (data);
 	    }
-
-	  gtk_widget_destroy (fs);
 	}
 
       /* Save button */
       else if (ret == 200)
 	{
-	  GtkWidget *fs;
 	  char s[1024];
+	  char *file;
 
 	  if (strcmp
 	      (gtk_entry_get_text (GTK_ENTRY (password)),
 	       gtk_entry_get_text (GTK_ENTRY (password2))))
 	    {
-	      gtk_dialog_msg (_("Passwords are different!"));
+	      dialog_msg (_("Passwords are different!"));
 	      continue;
 	    }
 
 	  snprintf (s, 1024, "%s %s - Save shoutcast config file", NAME,
 		    VERSION);
-	  fs =
-	    gtk_file_chooser_dialog_new (s, GTK_WINDOW (winmain),
-					 GTK_FILE_CHOOSER_ACTION_SAVE,
-					 GTK_STOCK_CANCEL,
-					 GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE,
-					 GTK_RESPONSE_ACCEPT, NULL);
 
-	  if (gtk_dialog_run (GTK_DIALOG (fs)) == GTK_RESPONSE_ACCEPT)
+	  if ((file =
+	       file_chooser (s, GTK_SELECTION_SINGLE,
+			     GTK_FILE_CHOOSER_ACTION_SAVE)))
 	    {
 	      FILE *fl;
-	      char *file =
-		(char *)
-		gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fs));
-
-	      gtk_widget_destroy (fs);
 
 	      if (dialog_test_file (file))
 		{
@@ -596,12 +582,16 @@ on_shoutcast_new_activate (GtkMenuItem * menuitem, gpointer user_data)
 
 	      fclose (fl);
 
-	      gtk_dialog_msg (_("Shoutcast config saved!"));
+	      dialog_msg (_("Shoutcast config saved!"));
 	    }
-
-	  else
-	    gtk_widget_destroy (fs);
 	}
+
+      /* Cancel button */
+      else
+	{
+	  break;
+	}
+
     }
 
   gtk_widget_destroy (dialog);
