@@ -1,4 +1,4 @@
-/* Somaplayer - Copyright (C) 2003-4 bakunin - Andrea Marchesini 
+/* Somaplayer - Copyright (C) 2003-5 bakunin - Andrea Marchesini 
  *                                     <bakunin@autistici.org>
  *
  * This source code is free software; you can redistribute it and/or
@@ -221,7 +221,8 @@ streaming_quit (audio * output)
 }
 
 int
-streaming_write (audio * output, int channels, int bitrate, void *buffer, size_t length)
+streaming_write (audio * output, int channels, int bitrate, void *buffer,
+		 size_t length)
 {
   audio_data_streaming *data = (audio_data_streaming *) output->data;
 
@@ -352,7 +353,7 @@ streaming_audio_device (audio * output)
       streaming->connect = HTTP;
 #ifdef ENABLE_OPENSSL
       streaming->bio = NULL;
-#endif      
+#endif
       streaming->fd = 0;
 
       len = strlen (output->audio_dev);
@@ -664,8 +665,7 @@ streaming_audio_device (audio * output)
 
 		  ch += 7;
 
-		  server =
-		    audio_parse_str (output->audio_dev, &ch, len);
+		  server = audio_parse_str (output->audio_dev, &ch, len);
 
 		  if (!strncmp (server, "http://", 7))
 		    {
@@ -901,4 +901,154 @@ streaming_socket_read (audio * ao, char *output, int ret)
     return read (data->fd, output, ret);
 }
 
+void
+streaming_update (audio * output)
+{
+  audio_data_streaming *data = (audio_data_streaming *) output->data;
+  int fd;
+  int len = 0;
+
+#ifdef ENABLE_OPENSSL
+  SSL *ssl;
+  SSL_CTX *ctx;
+  BIO *bio;
+#endif
+
+  struct sockaddr_in sock;
+  struct hostent *hp;
+
+  char buffer[SIZE_BUFFER];
+  char *metadata;
+
+  msg (_("Update metadata."));
+
+  if(!(metadata = html_encode(playing_name ())))
+    return;
+
+  if (0);
+
+#ifdef ENABLE_SHOUTCAST
+  else if (output->audio_type == USE_SHOUTCAST)
+    len =
+      snprintf (buffer, sizeof (buffer),
+		"GET /admin.cgi?mode=updinfo&pass=%s&song=%s HTTP/1.0\r\n"
+		"User-Agent: %s/%s\r\n\r\n", data->password, metadata, NAME,
+		VERSION);
+#endif
+
+#ifdef ENABLE_ICECAST2
+  else if (output->audio_type == USE_ICECAST2)
+    {
+      char t[1024];
+
+      snprintf (t, sizeof (t), "source:%s", data->password);
+      len =
+	snprintf (buffer, sizeof (buffer),
+		  "GET /admin/metadata?mode=updinfo&mount=/%s&song=%s HTTP/1.0\r\n"
+		  "User-Agent: %s/%s\r\n" "Authorization: Basic %s\r\n\r\n",
+		  data->mount, metadata, NAME, VERSION, base64 (t));
+    }
+#endif
+
+  else
+    len =
+      snprintf (buffer, sizeof (buffer),
+		"GET /admin.cgi?mode=updinfo&pass=%s&mount=/%s&song=%s HTTP/1.0\r\n"
+		"User-Agent: %s/%s\r\n\r\n", data->password, data->mount,
+		metadata, NAME, VERSION);
+
+  free(metadata);
+	
+  if (!len)
+    return;
+
+  if (data->connect == HTTPS)
+    {
+#ifdef ENABLE_OPENSSL
+      char *tmp;
+
+      SSLeay_add_ssl_algorithms ();
+      ctx = SSL_CTX_new (SSLv23_client_method ());
+      bio = BIO_new_ssl_connect (ctx);
+      BIO_get_ssl (bio, &ssl);
+
+      if (!ssl)
+	{
+	  msg_error (_("Ssl connection error."));
+	  return;
+	}
+
+      SSL_set_mode (ssl, SSL_MODE_AUTO_RETRY);
+
+      if (!
+	  (tmp =
+	   (char *) malloc ((strlen (data->server) + 7) * sizeof (char))))
+	fatal (_("Error: memory."));
+
+      sprintf (tmp, "%s:%d", data->server, data->port);
+
+      if (BIO_set_conn_hostname (bio, tmp) < 0)
+	{
+	  msg_error (_("Ssl connection error."));
+	  free (tmp);
+	  return;
+	}
+
+      if (BIO_do_connect (bio) < 0)
+	{
+	  msg_error (_("Ssl connection error."));
+	  free (tmp);
+	  return;
+	}
+
+      if (BIO_do_handshake (bio) < 0)
+	{
+	  msg_error (_("Ssl connection error."));
+	  free (tmp);
+	  return;
+	}
+
+      free (tmp);
+
+      BIO_write (bio, buffer, len);
+      BIO_ssl_shutdown (bio);
+      BIO_free_all (bio);
+
+#else
+      msg_error (_("No support for ssl connection. Compile with SSL."));
+      return;
+#endif
+    }
+
+  else
+    {
+
+      if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
+	{
+	  msg_error (_("Socket error."));
+	  return;
+	}
+
+      if (!(hp = gethostbyname (data->server)))
+	{
+	  msg_error (_("Error: unknow host."));
+	  return;
+	}
+
+      memset ((void *) &sock, 0, sizeof (sock));
+      sock.sin_family = AF_INET;
+      sock.sin_addr.s_addr = ((struct in_addr *) (hp->h_addr))->s_addr;
+      sock.sin_port = htons (data->port);
+
+      if (connect (fd, (struct sockaddr *) &sock, sizeof (sock)) < 0)
+	{
+	  msg_error (_("Connect error."));
+	  return;
+	}
+
+      write (fd, buffer, len);
+      close (fd);
+    }
+}
+
 #endif
